<?php

namespace App\Factory;

use App\Exception;

class BranchFactory {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getStandartVendor ($branch) {
        $sql = "
            SELECT CDVENDEDOR
              FROM PARAVEND
            WHERE CDFILIAL = :CDFILIAL";
            
        $params = array(
            ":CDFILIAL" => $branch    
        );
        
        $vendor = $this->connection->fetchAll($sql, $params);
        
        if(empty($vendor)) 
            throw new Exception\ParamsException("Não foi encontrado vendedor padrão para a filial - $branch", 101);
        
        if(!isset($vendor[0]["CDVENDEDOR"])) 
            throw new Exception\ParamsException("Não foi encontrado vendedor padrão para a filial - $branch", 101);
        
        return $vendor[0]["CDVENDEDOR"];
    }
    
    public function getStandartStore ($branch, $vendor) {
        $sql = "
            SELECT C.CDLOJA, C.IDIMPAUTENTREG AS IMPAUTO 
              FROM VENDEDOR V
              
              JOIN CAIXA C
                ON C.CDCAIXA  = V.CDCAIXA
              
              JOIN LOJA L 
                ON C.CDLOJA   = L.CDLOJA
               AND C.CDFILIAL = L.CDFILIAL
            
             WHERE V.CDVENDEDOR = :CDVENDEDOR
               AND C.CDFILIAL   = :CDFILIAL        
        ";
        
        $params = array(
            ":CDFILIAL"   => $branch,
            ":CDVENDEDOR" => $vendor
        );
        
        $store = $this->connection->fetchAll($sql, $params);
        
        if(empty($store)) 
            throw new Exception\ParamsException("Não foi encontrada loja para a filial - $branch", 102);
        
        if(!isset($store[0]["CDLOJA"])) 
            throw new Exception\ParamsException("Não foi encontrada loja para a filial - $branch", 102);
        
        return $store[0];
    }
    
    public function getStandartCashier ($branch, $vendor) {
        $sql = "
            SELECT CDCAIXA, CDOPERADOR
              FROM VENDEDOR
             WHERE CDFILIAL   = :CDFILIAL
               AND CDVENDEDOR = :CDVENDEDOR";
        
        $params = array(
            ":CDFILIAL"   => $branch,
            ":CDVENDEDOR" => $vendor
        );
        
        $cashier = $this->connection->fetchAll($sql, $params);
        
        if(empty($cashier)) 
            throw new Exception\ParamsException("Não foi encontrado caixa associado ao vendedor padrão - $vendor | filial - $branch", 103);
        
        if(!isset($cashier[0]["CDCAIXA"])) 
            throw new Exception\ParamsException("Não foi encontrado caixa associado ao vendedor padrão - $vendor | filial - $branch", 103);
        
        return $cashier[0];
    }
    
    public function getProductControlId ($branch, $store) {
        $sql = "
            SELECT IDCONTROPROD
              FROM LOJA
            WHERE CDFILIAL = :CDFILIAL
              AND CDLOJA = :CDLOJA";
     
        $params = array(
            ":CDFILIAL" => $branch,
            ":CDLOJA"   => $store
        );
        
        $idControlProd = $this->connection->fetchAll($sql, $params);
        
        if (empty($idControlProd)) 
            throw new Exception\ParamsException("A tela de controle de produção não está parametrizada para a filial - $branch", 104);
        
        if (!isset($idControlProd[0]["IDCONTROPROD"]))
            throw new Exception\ParamsException("A tela de controle de produção não está parametrizada para a filial - $branch", 104);
        
        return $idControlProd[0]["IDCONTROPROD"];
    }
    
    public function getStandartClient ($branch) {
        $sql = "
            SELECT CDCLIENTE
              FROM PARAVEND
            WHERE CDFILIAL = :CDFILIAL";
        
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        $stdClient = $this->connection->fetchAll($sql, $params)[0];
        
        if(empty($stdClient["CDCLIENTE"])) 
            throw new Exception\ParamsException("Não foi encontrado cliente padrão para a filial - $branch", 105);
        
        return $stdClient;
    }
    
    public function getStandartConsumerType ($branch) {
        $sql = "
            SELECT CDTIPOCONS, CDCCUSCLIE
              FROM PARAVEND
            WHERE CDFILIAL = :CDFILIAL";
        
        $params = array(
            ":CDFILIAL" => $branch    
        );
        
        $consumerT = $this->connection->fetchAll($sql, $params)[0];
        
        if(empty($consumerT["CDTIPOCONS"]))
            throw new Exception\ParamsException("Não foi encontrado tipo de consumidor padrão para a filial - $branch", 106);
        
        if(empty($consumerT["CDCCUSCLIE"]))
            throw new Exception\ParamsException("Não foi encontrado centro de custo para a filial - $branch", 106);
        
        return $consumerT;
    }
    
    public function getBranchCityCode ($branch) {
        $sql = "
            SELECT CDMUNICIPIO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL";
        
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        return $this->connection->fetchAll($sql, $params)[0]["CDMUNICIPIO"];
    }
    
    public function getBranchState ($branch) {
        $sql = "
            SELECT SGESTADO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL
        ";
        
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        return $this->connection->fetchAll($sql, $params)[0]["SGESTADO"];
    }
    
    public function getBranchAddress($branch){
        $sql = "
                SELECT
                      EF.CDFILIAL, EF.CDPAIS, EF.SGESTADO, EF.CDMUNICIPIO,
                      EF.DSENDEFILI, EF.NMBAIRFILI, EF.NRCEPFILI
                FROM  ENDEFILI EF
                WHERE EF.CDFILIAL = :CDFILIAL
        ";
        
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        return $this->connection->fetchAssoc($sql, $params);
        
    }
        
    public function getBranchs(){
        $sql = "
            SELECT IF.CDINTEGRACAO, IF.CDFILIAL, IF.IDTOKEN, IF.IDATIVO,
            IF.IDUTLENTRETERC, IF.CDLOJA, IF.CDCAIXA, IF.AUTHTOKEN, IF.AUTHTKNDTCRIAC,
            NVL(IF.IDTOKENINTGERALFILI, IT.IDTOKENINTGERAL) AS IDTOKENINTGERAL,
            NVL(IF.IDSENHATOKENGERALFILI, IT.IDSENHATOKENGERAL) AS IDSENHATOKENGERAL 
            FROM INTEGRACAOFILI IF
            LEFT JOIN INTEGRACAO IT
            ON IT.CDINTEGRACAO = IF.CDINTEGRACAO
            WHERE IT.IDINTEGRACAO = 'DDI'
        ";
        
        try{
            return $branchIds =  $this->connection->fetchAll($sql, []);
        } catch(\Exception $e){
            return [
                'error' => true,
                'message' => 'Erro ao buscar areas de atendimento.Erro: '.$e->getMessage()
            ];
        }
    }
}
