<?php

namespace App\Factory;

use App\Exception;

class ConsumerFactory {
    
    private $connection;
    
    public function __construct(\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }
    
    public function getCityCode($state, $city) {
        $sql = "
            SELECT CDMUNICIPIO 
              FROM MUNICIPIO
             WHERE UPPER(SGESTADO) = :SGESTADO
               AND CDPAIS   = :CDPAIS
               AND NVL((SELECT CDMUNICIPIO
                        FROM MUNICIPIO
                       WHERE UPPER(NMMUNICIPIO) = :NMMUNICIPIO
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
                         AND ROWNUM = 1),
                     (SELECT CDMUNICIPIO
                        FROM MUNICIPIO
                       WHERE UPPER(NMMUNICIPIO) LIKE '%'||:NMMUNICIPIO||'%'
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
                         AND ROWNUM = 1)) = CDMUNICIPIO 
        ";
            
        $params = [
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => strtoupper($state),
            ":NMMUNICIPIO" => strtoupper($city)
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result)) return null;
        
        return $result[0]["CDMUNICIPIO"];
    }
    
    public function getNeighborhoodCode($addressInfo, $cityCode) {
        $sql = "
            SELECT CDBAIRRO
              FROM BAIRRO
            WHERE CDPAIS          = :CDPAIS
              AND SGESTADO        = :SGESTADO
              AND CDMUNICIPIO     = :CDMUNICIPIO
              AND UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)";
        
        $params = [
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => $addressInfo["state"],
            ":CDMUNICIPIO" => $cityCode,
            ":NMBAIRRO"    => $this->util->removeAccents($addressInfo["neighborhood"])
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result)) 
            return null;
        
        return $result[0]["CDBAIRRO"];
    }
    
    public function registerConsumer($consumerDetails) {

        $sql = "
            INSERT INTO CONSUMIDOR (
                CDCLIENTE, CDCONSUMIDOR, NMCONSUMIDOR, IDSITCONSUMI,
                CDCCUSCLIE, NRCPFRESPCON, IDCONSUMIDOR, IDATUCONSUMI, IDTPVENDACONS,
                IDIMPCPFCUPOM, IDCADCONFLIBCON, IDPERCONSPRODEX, IDTPSELMANHA,
                IDTPSEALMOCO, IDTPSELTARDE, IDCRACHAMESTRE, CDPAIS,
                SGESTADO, CDMUNICIPIO, CDTIPOCONS, CDIDCONSUMID,
                DSENDECONS, NMBAIRCONS, NRCEPCONS, NRTELECONS,
                DSEMAILCONS, NMRESPCONS, DSREFERENDECONS, DSINFOCONS, IDCONSEXT 
            )
            VALUES (
                :CDCLIENTE, :CDCONSUMIDOR, :NMCONSUMIDOR, :IDSITCONSUMI,
                :CDCCUSCLIE, :NRCPFRESPCON, :IDCONSUMIDOR, :IDATUCONSUMI, :IDTPVENDACONS,
                :IDIMPCPFCUPOM, :IDCADCONFLIBCON, :IDPERCONSPRODEX, :IDTPSELMANHA,
                :IDTPSEALMOCO, :IDTPSELTARDE, :IDCRACHAMESTRE, :CDPAIS,
                :SGESTADO, :CDMUNICIPIO, :CDTIPOCONS, :CDIDCONSUMID,
                :DSENDECONS, :NMBAIRCONS, :NRCEPCONS, :NRTELECONS,
                :DSEMAILCONS, :NMRESPCONS, :DSREFERENDECONS, :DSINFOCONS, :IDCONSEXT
            )
        ";
        
        $params = [
            ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
            ":NMCONSUMIDOR"    => substr($consumerDetails["customer"]["firstName"] . " " . $consumerDetails["customer"]["lastName"], 0, 49),
            ":IDSITCONSUMI"    => "1",
            ":CDCCUSCLIE"      => $consumerDetails["CDCCUSCLIE"],
            ":NRCPFRESPCON"    => null,
            ":IDCONSUMIDOR"    => "C",
            ":IDATUCONSUMI"    => "S",
            ":IDTPVENDACONS"   => "7", // NOT NULL
            ":IDIMPCPFCUPOM"   => "S", // NOT NULL
            ":IDCADCONFLIBCON" => "N", // NOT NULL
            ":IDPERCONSPRODEX" => "S", // NOT NULL
            ":IDTPSELMANHA"    => "N", // NOT NULL
            ":IDTPSEALMOCO"    => "N", // NOT NULL
            ":IDTPSELTARDE"    => "N", // NOT NULL
            ":IDCRACHAMESTRE"  => "N", // NOT NULL
            ":CDPAIS"          => "0055", // FIXED COUNTRY CODE
            ":SGESTADO"        => $consumerDetails["address"]["state"],
            ":CDMUNICIPIO"     => $consumerDetails["CDMUNICIPIO"],
            ":CDTIPOCONS"      => $consumerDetails["CDTIPOCONS"],
            ":CDIDCONSUMID"    => $consumerDetails["CDCONSUMIDOR"],
            ":DSENDECONS"      => substr($consumerDetails["address"]["street"] . " " .  $consumerDetails["address"]["number"], 0, 49),
            ":NMBAIRCONS"      => substr($consumerDetails["address"]["neighborhood"], 0, 29),
            ":NRCEPCONS"       => preg_replace("/[^0-9]/", "", $consumerDetails["address"]["zipcode"]),
            ":NRTELECONS"      => $this->sanatizeConsumerPhone($consumerDetails["customer"]["telephone"]),
            ":DSEMAILCONS"     => substr($consumerDetails["customer"]["email"], 0, 49),
            ":NMRESPCONS"      => substr($consumerDetails["customer"]["firstName"] . " " . $consumerDetails["customer"]["lastName"], 0, 49),
            ":DSREFERENDECONS" => $consumerDetails["address"]["street"] .", ".  $consumerDetails["address"]["number"],
            ":DSINFOCONS"      => "SDE - Incluido",
            ":IDCONSEXT"       => null
        ];
        
        $this->connection->executeQuery($sql, $params);
        
        $sql = "
            SELECT *
              FROM CONSUMIDOR
             WHERE CDCLIENTE    = :CDCLIENTE 
               AND CDCONSUMIDOR = :CDCONSUMIDOR 
        ";
            
        $params = [
            ":CDCLIENTE"    => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumerDetails["CDCONSUMIDOR"]
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getConsumerByTelNumber($telNumber) {
        $sql = "
            SELECT CO.CDCLIENTE, CO.CDCONSUMIDOR,
			   CO.NMCONSUMIDOR, CO.NRCPFRESPCON,
			   CO.CDTIPOCONS, CO.NRRGCONSUMID,
			   CO.DSEMAILCONS, CO.DTNASCCONS,
			   CO.NRCEPCONS, CO.DSEXIBENDE, 'PRINCIPAL' AS CDENDECONS,
	 		   CO.CDPAIS, CO.DSENDECONS,
	 		   CO.NRENDECONS, CO.DSCOMPLENDECONS,
	 		   CO.NRTELECONS, CO.NRCELULARCONS,
	 		   CO.DSREFERENDECONS,
	 		   CO.CDCONSFACEBOOK, CO.DSENDEIMGCONS,
	 		   CO.CDTOKENCONSFB, CO.DTEXPTOKENFB,
	 		   CO.NRLATCONS, CO.NRLONGCONS, CO.IDCONSEXT
		    FROM CONSUMIDOR CO
		    WHERE NRTELECONS = :NRTELECONS";
		    
		$params = [
            ":NRTELECONS" => preg_replace("/[^0-9]/", "", $telNumber)
		];
		
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function formatTaxPayerId($id) {
        $id = preg_replace("/[^0-9]/", "", $id);
        return (strlen($id) <= 11) ? $id : "";
    }
    
    public function updateIdConsExt($consumerDetails) {
        $sql = "
            UPDATE CONSUMIDOR 
               SET IDCONSEXT = :IDCONSEXT 
             WHERE CDCLIENTE = :CDCLIENTE 
               AND CDCONSUMIDOR = :CDCONSUMIDOR";
        
        $params = [
            ":CDCLIENTE" => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumerDetails["CDCONSUMIDOR"],
            ":IDCONSEXT" => $consumerDetails["IDCONSEXT"]
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getConsumerByidConsExt($idConsExt) {
        $sql = "
            SELECT CO.CDCLIENTE, CO.CDCONSUMIDOR,
			   CO.NMCONSUMIDOR, CO.NRCPFRESPCON,
			   CO.CDTIPOCONS, CO.NRRGCONSUMID,
			   CO.DSEMAILCONS, CO.DTNASCCONS,
			   CO.NRCEPCONS, CO.DSEXIBENDE, 'PRINCIPAL' AS CDENDECONS,
	 		   CO.CDPAIS, CO.DSENDECONS,
	 		   CO.NRENDECONS, CO.DSCOMPLENDECONS,
	 		   CO.NRTELECONS, CO.NRCELULARCONS,
	 		   CO.DSREFERENDECONS,
	 		   CO.CDCONSFACEBOOK, CO.DSENDEIMGCONS,
	 		   CO.CDTOKENCONSFB, CO.DTEXPTOKENFB,
	 		   CO.NRLATCONS, CO.NRLONGCONS, CO.IDCONSEXT
		    FROM CONSUMIDOR CO
		    WHERE IDCONSEXT = :IDCONSEXT";
		    
		$params = [
            ":IDCONSEXT" => $idConsExt
		];
		
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getConsumerByTaxPayer($taxPayerIdentificationNumber) {
        $sql = "
            SELECT CO.CDCLIENTE, CO.CDCONSUMIDOR,
			   CO.NMCONSUMIDOR, CO.NRCPFRESPCON,
			   CO.CDTIPOCONS, CO.NRRGCONSUMID,
			   CO.DSEMAILCONS, CO.DTNASCCONS,
			   CO.NRCEPCONS, CO.DSEXIBENDE, 'PRINCIPAL' AS CDENDECONS,
	 		   CO.CDPAIS, CO.DSENDECONS,
	 		   CO.NRENDECONS, CO.DSCOMPLENDECONS,
	 		   CO.NRTELECONS, CO.NRCELULARCONS,
	 		   CO.DSREFERENDECONS,
	 		   CO.CDCONSFACEBOOK, CO.DSENDEIMGCONS,
	 		   CO.CDTOKENCONSFB, CO.DTEXPTOKENFB,
	 		   CO.NRLATCONS, CO.NRLONGCONS, CO.IDCONSEXT
		    FROM CONSUMIDOR CO
		    WHERE NRCPFRESPCON = :NRCPFRESPCON";
		    
		$params = [
            ":NRCPFRESPCON" => preg_replace("/[^0-9]/", "", $taxPayerIdentificationNumber)
		];
		
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getConsumerByEmail($email) {
        $sql = "
            SELECT CO.CDCLIENTE, CO.CDCONSUMIDOR, 
                CO.NMCONSUMIDOR, CO.NRCPFRESPCON, 
                CO.CDTIPOCONS, CO.NRRGCONSUMID, 
                CO.DSEMAILCONS, CO.DTNASCCONS, 
                CO.NRCEPCONS, CO.DSEXIBENDE, 'PRINCIPAL' AS CDENDECONS, 
                CO.CDPAIS, CO.DSENDECONS, 
                CO.NRENDECONS, CO.DSCOMPLENDECONS, 
                CO.NRTELECONS, CO.NRCELULARCONS, 
                CO.DSREFERENDECONS, 
                CO.CDCONSFACEBOOK, CO.DSENDEIMGCONS, 
                CO.CDTOKENCONSFB, CO.DTEXPTOKENFB, 
                CO.NRLATCONS, CO.NRLONGCONS, CO.IDCONSEXT 
            FROM CONSUMIDOR CO 
            WHERE UPPER(DSEMAILCONS) = UPPER(:DSEMAILCONS) 
        ";
        
		$params = [
            ":DSEMAILCONS" =>substr($email, 0, 49)
		];
		
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    private function sanatizeConsumerPhone($number) {
        return preg_replace("/[^0-9]/", "", $number);
    }
}
