<?php

namespace App\Factory;

use App\Exception;

class OrderFactory {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }
    
    public function insertSale ($sale, $generalParams, $consumer) {
        $sql = "
            INSERT INTO VENDAREST
    			(CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
    			DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
    			CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
    			:CDVENDEDOR, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'), null, :CDOPERADOR,
    			:NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, 'S')";
    			
    	date_default_timezone_set($this->util->getCurrentTimezone(strtoupper($generalParams["SGESTADO"])));
    	
    	$params = [
    	    ":CDFILIAL"     => $generalParams["CDFILIAL"],
            ":NRVENDAREST"  => $sale,
            ":CDLOJA"       => $generalParams["CDLOJA"],
            ":NRMESA"       => "",
            ":CDVENDEDOR"   => $generalParams["CDVENDEDOR"],
            ":CURRDATE"     => date("d/m/Y H:i:s"),
            ":CDOPERADOR"   => $generalParams["CDOPERADOR"],
            ":NRPESMESAVEN" => "1",
            ":CDCLIENTE"    => $consumer["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumer["CDCONSUMIDOR"]    
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertSaleCommand ($generalParams, $order, $consumer) {
        
        $sql = "
            INSERT INTO COMANDAVEN 
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRCOMANDAEXT, 
    	        CDLOJA, DSCOMANDA, IDSTCOMANDA, SGSEXOCON, 
    	        TXMOTIVCANCE, VRACRCOMANDA, VRDESCOMANDA, CDPAIS, SGESTADO, 
    	        CDMUNICIPIO, NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO, NRENDECONSCOMAN,
    	        DSENDECONSCOMAN, DSCOMPLENDCOCOM, DSREFENDCONSCOM, DSOBSCOMANDA,
    	        IDORGCMDVENDA, IDRETBALLOJA, NRPEDMOBILE, NRLATPEDIDO, NRLONGPEDIDO, 
    	        DSAREAATEND)
    	    VALUES 
    	        (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRCOMANDAEXT, 
    	        :CDLOJA, :DSCOMANDA, :IDSTCOMANDA, :SGSEXOCON, 
    	        :TXMOTIVCANCE, :VRACRCOMANDA, :VRDESCOMANDA, :CDPAIS, :SGESTADO, 
    	        :CDMUNICIPIO, :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO, :NRENDECONSCOMAN,
    	        :DSENDECONSCOMAN, :DSCOMPLENDCOCOM, :DSREFENDCONSCOM, :DSOBSCOMANDA,
    	        :IDORGCMDVENDA, :IDRETBALLOJA, :NRPEDMOBILE, :NRLATPEDIDO,
    	        :NRLONGPEDIDO, :DSAREAATEND)";
    	
    	$params = [
    	    ":CDFILIAL"        => $generalParams["CDFILIAL"],
            ":NRVENDAREST"     => $order["NRVENDAREST"],
            ":NRCOMANDA"       => $order["NRCOMANDA"],
            ":NRCOMANDAEXT"    => $order["orderNumber"],
            ":CDLOJA"          => $generalParams["CDLOJA"],
            ":DSCOMANDA"       => "DLV_".$order["NRCOMANDA"],
            ":IDSTCOMANDA"     => $order["IDSTCOMANDA"],
            ":SGSEXOCON"       => "M",
            ":TXMOTIVCANCE"    => "",
            ":VRACRCOMANDA"    => $this->util->tofloat($order["total"]["deliveryFee"]["value"]),
            ":VRDESCOMANDA"    => 0,
            ":CDPAIS"          => "0055",
            ":SGESTADO"        => $order["address"]["state"], 
            ":CDMUNICIPIO"     => $consumer["CDMUNICIPIO"],
            ":NRCEPCONSCOMAND" => $order["address"]["zipcode"],
            ":CDBAIRRO"        => $consumer["CDBAIRRO"],
            ":DSBAIRRO"        => substr($order["address"]["neighborhood"], 0, 99),
            ":NRENDECONSCOMAN" => empty($order["address"]["number"]) ? "" : $order["address"]["number"],
            ":DSENDECONSCOMAN" => substr($order["address"]["street"], 0, 49),
            ":DSCOMPLENDCOCOM" => (isset($order["address"]["complement"]) && !empty($order["address"]["complement"])) ? $order["address"]["complement"] : "",
            ":DSREFENDCONSCOM" => (isset($order["address"]["reference_point"]) && !empty($order["address"]["reference_point"])) ? $order["address"]["reference_point"] : "",
            ":DSOBSCOMANDA"    => isset($generalParams['DSOBSCOMANDA']) ? $generalParams['DSOBSCOMANDA'] : "",
            ":IDORGCMDVENDA"   => "DLV_DDI",
            ":IDRETBALLOJA"    => $order["type"] === "TAKEOUT" ? "S" : "N",
            ":NRPEDMOBILE"     => "DDI".$order["orderNumber"],
            ":NRLATPEDIDO"     => null,
            ":NRLONGPEDIDO"    => null,
            ":DSAREAATEND"     => $consumer["DSAREAATEND"]
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertCashMoviment ($order, $value, $consumer) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";

        $params = [
            ":CDFILIAL"        => $order["branch"],
            ":NRVENDAREST"     => $order["NRVENDAREST"],
            ":NRSEQMOVDLV"     => $order["NRSEQMOVDLV"],
            ":IDTIPOMOVIVEDLV" => "E",
            ":CDCLIENTE"       => $consumer["CDCLIENTE"],
            ":CDCONSUMIDOR"    => $consumer["CDCONSUMIDOR"],
            ":CDTIPORECE"      => $order["CDTIPORECE"],
            ":VRMOVIVENDDLV"   => $this->util->tofloat($value),
            ":NRCARTBANCODLV"  => null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function insertChange ($order, $value, $generalParams, $consumer, $movCaixa, $cdTipoRece) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";
        
        $params = [
            ":CDFILIAL"        => $order["branch"],
            ":NRVENDAREST"     => $order["NRVENDAREST"],
            ":NRSEQMOVDLV"     => $movCaixa,
            ":IDTIPOMOVIVEDLV" => "S",
            ":CDCLIENTE"       => $consumer["CDCLIENTE"],
            ":CDCONSUMIDOR"    => $consumer["CDCONSUMIDOR"],
            ":CDTIPORECE"      => $cdTipoRece,
            ":VRMOVIVENDDLV"   => $this->util->toFloat($value),
            ":NRCARTBANCODLV"  => null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function validateDuplication ($order, $branch) {
        $sql = "
            SELECT C.NRPEDMOBILE
    		  FROM COMANDAVEN C
    		  JOIN VENDAREST V
    		    ON V.NRVENDAREST = C.NRVENDAREST
    		   AND V.CDFILIAL = C.CDFILIAL
    		 WHERE C.NRPEDMOBILE   = :NRPEDMOBILE
    		   AND V.CDFILIAL      = :CDFILIAL
    		   AND C.IDORGCMDVENDA = :IDORGCMDVENDA
    		   AND DTHRABERMESA >= SYSDATE - 1
        ";
        
        $params = [
            ":CDFILIAL"    => $branch,
            ":NRPEDMOBILE" => "DDI".$order,
            ":IDORGCMDVENDA" => 'DLV_DDI'
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        return empty($result);
    }
    
    public function getOrderForCancel ($orderReference) {
        $sql = "
            SELECT C.CDFILIAL branchId, C.NRCOMANDA orderNumber, C.NRVENDAREST sellNumber, C.IDSTCOMANDA orderStatus
              FROM COMANDAVEN C 
         LEFT JOIN VENDA V
                ON C.CDFILIAL = V.CDFILIAL
               AND C.NRVENDAREST = V.NRVENDAREST
               AND V.NRSEQVENDA IS NULL
             WHERE C.NRPEDMOBILE   = :NRPEDMOBILE
               AND C.IDORGCMDVENDA = :IDORGCMDVENDA
               AND C.IDSTCOMANDA <> '4'
        ";
        
        $params = [
            ':NRPEDMOBILE'   => "DDI".$orderReference,
            ':IDORGCMDVENDA' => 'DLV_DDI'
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function cancelOrder ($cancelOrder) {
        $sql = "
            UPDATE COMANDAVEN
		       SET IDSTCOMANDA = '4'
		     WHERE CDFILIAL = :CDFILIAL
	           AND NRCOMANDA = :NRCOMANDA
	           AND NRVENDAREST = :NRVENDAREST";
        
		$params = [
			':CDFILIAL'	 => $cancelOrder["BRANCHID"], 
			':NRCOMANDA' => $cancelOrder["ORDERNUMBER"],
			':NRVENDAREST' => $cancelOrder["SELLNUMBER"]
		];
		
		$this->connection->executeQuery($sql, $params);
		
		$this->syncOrder($cancelOrder);
	}
	
    public function syncOrder ($orderDetails) {
        $sql = "
            UPDATE VENDAREST
               SET IDSINCMSDE = 'N', NRVENDARESTAUX = NULL
            WHERE CDFILIAL = :CDFILIAL
                AND NRVENDAREST = :NRVENDAREST
        ";
        
        $params = [
            ':CDFILIAL' => $orderDetails["BRANCHID"],
            ':NRVENDAREST' => $orderDetails["SELLNUMBER"]
       ];
       
       $this->connection->executeQuery($sql, $params);
	}
	
    public function validateOperator($generalParams){
        $sql = "
            SELECT * FROM FILIOPER
             WHERE CDOPERADOR   = :CDOPERADOR
               AND CDFILIAL     = :CDFILIAL
        ";
        
        $params = [
            ":CDFILIAL"   => $generalParams["CDFILIAL"],
            ":CDOPERADOR" => $generalParams["CDOPERADOR"]
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
        
    }
    
    public function fetchIsCashierSinc($filial){
        $sql = "
        SELECT  IDSINCCAIXADLV 
          FROM  CAIXA
          WHERE CDFILIAL = :CDFILIAL AND
                UPPER(IDSINCCAIXADLV) = 'S'
        ";
        $params = [
            ":CDFILIAL" => $filial
        ];
        
        return $this->connection->fetchAssoc($sql,$params);
    }
    
    //Prevents teknisa from crazy discount logics by partners ¯\_(ツ)_/¯
    public function validateOrder($cdFilial, $nrVendaRest) {

        $taxSQL = "
            SELECT VRACRCOMANDA, VRDESCOMANDA 
              FROM COMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $itemsSQL = "
            SELECT QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $entradasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'E'
        ";

        $saidasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'S'
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];

        $taxData      = $this->connection->fetchAssoc($taxSQL, $params);
        $itemsData    = $this->connection->fetchAll($itemsSQL, $params);
        $entradasData = $this->connection->fetchAll($entradasSQL, $params);
        $saidasData   = $this->connection->fetchAll($saidasSQL, $params);

        $discountSum = isset($taxData['VRDESCOMANDA']) ? (string)$this->util->formatNumber($taxData['VRDESCOMANDA']) : "0.00";
        $taxSum      = isset($taxData['VRACRCOMANDA']) ? (string)$this->util->formatNumber($taxData['VRACRCOMANDA']) : "0.00";
        $itemsSum    = "0.00";
        $entradasSum = "0.00";
        $saidasSum   = "0.00";

        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->util->formatNumber($itemData['VRPRECCOMVEN']), $this->util->formatNumber($itemData['QTPRODCOMVEN']), 2);
            $sub      = bcsub($mul, $this->util->formatNumber($itemData['VRDESCCOMVEN']), 2);
            $itemsSum = bcadd($itemsSum, $sub, 2);
        }

        foreach($entradasData as $entradaData) {
            $entradasSum = bcadd($entradasSum, $this->util->formatNumber($entradaData['VRMOVIVENDDLV']), 2);
        }

        foreach($saidasData as $saidaData) {
            $saidasSum = bcadd($saidasSum, $this->util->formatNumber($saidaData['VRMOVIVENDDLV']), 2);
        }

        $subOrderSum   = bcadd($itemsSum, $taxSum, 2);
        $totalOrderSum = bcsub($subOrderSum, $discountSum, 2);
        $paymentsSum   = bcsub($entradasSum, $saidasSum, 2);
        
        if(bccomp($totalOrderSum, "0", 2) <= 0 || bccomp($paymentsSum, "0", 2) <= 0) {
            throw new Exception\IntegrationException("Não foi possível processar o pedido, desconto de 100% ou valor pago ZERADO.<br>Descontos ($discountSum) - Total de Itens ($subOrderSum) = $totalOrderSum | Valor pago = $paymentsSum");
        }
        
        if(bccomp($totalOrderSum, $paymentsSum) != 0) {
            throw new Exception\IntegrationException("Não foi possível processar o pedido, existe uma divergencia entre os itens do pedido e o valor pago.<br>Valor dos itens: $totalOrderSum | Valor pago: $paymentsSum");
        }
    }
}
